package article_test

import (
	"encoding/json"
	"fmt"
	"os"
	"time"

	"github.com/lestrrat-go/jwx/v2/jwa"
	"github.com/lestrrat-go/jwx/v2/jwk"
	"github.com/lestrrat-go/jwx/v2/jwt"
)

func init() {
	// "aud"にひとつだけ要素があった時にを配列ではなく
	// 文字列として表記するため
	jwt.Settings(jwt.WithFlattenAudience(true))
}

func Example() {
	fmt.Println("JWTの作成")

	// 再現性があるように「現在時刻」は固定
	now := time.Unix(1679901069, 0)

	token, err := jwt.NewBuilder().
		Issuer("gihyo.jp/wd-press/when-ingo").
		IssuedAt(now).
		Audience([]string{"wdpress-readers"}).
		Subject("sample").
		Build()
	if err != nil {
		fmt.Printf("failed to build token: %s\n", err)
		return
	}

	enc := json.NewEncoder(os.Stdout)
	enc.Encode(token)

	fmt.Println("JWKの作成")

	// 再現性があるように、文字列として保存して鍵を作る。毎回違う鍵を
	// 使う場合は rsa.GenerateKey(rand.Reader, 2048)などでRSA鍵を得たのち
	// jwk.FromRw()などでJWKを得る
	const privKeySrc = `{"d":"QecTOvJSEUkdmi-4oolu5UM2jbcsgdZgYou5OarsbubLaQOBOb3u38fr_eWkIiB4x0atVScC6PD2nldQlOcE9mQhhyumoSAYGZ-u4yU7PIrn23hYw7poLXzSbxeXtrRNSE6CMo36IMZFTpTA7Km5ur8wRVLYyjpz1YZG7zZiHlxeCH9xyuvrScCefFwI96fL0MrX56x0-ZMXX65pByZLtAYJmKFuep7u8UlJaR9f0PIlL78JkioveqFPrvzxI2LN3_BGagHDgC75vvC0m9DqfJ3xCoLoasFehPPrfuKk2mZm28Yp4mth7mL7zUgoe0UhHVEtipmIAdqo_7INZOg1TQ","dp":"sKtTWmygW9kYAUUzTV63SKj2sZKZkD9jBjOomV_VMo3AlzQSEp-mmOV5UTojlStCf_Ug9_AXMYMc0kxFagZaqI1CvOnW3z12CGi-hbHMEsmEiCZ0wmJZu4-KwbiQFLM_m7CL-y66Jt7pQ4t1BkYs67bTyzUGLC_vrkkR8Zu0vW8","dq":"hE7aGFysXuQlnynAj_458QpQiovAfxXcNLWlYre32yGvgm4GwOgMQOdD_epo4NJ48rf_csXXxVC8JKfzfObgHm4wNcbdasCR6Y8QlaXTDbu98JlwRU4D9O8kj-yNb56SYmSWnRkUZTRer1oUf-EgGEo1EQ8ho0IAK7htQRLSaXE","e":"AQAB","kty":"RSA","n":"qMyOIeSJRAlQSKZux2HGIzgOfyRPh87MF8Xtn-SG7UC0utgSdjtL2HzmsgG4nxY696kKu9a9Ww4Sn1w53x6LTgpibElqT87Jih4zUHjYXhv1TF7uOMKyq2_i0wcFiSjpH901FmxSY6yy0b9DHkWZaaIAoBzLttXURw6kW71hJZSNeUYP2V1WzYdRATcirLRyXKBVQDYWcgq2cblH9oxT2GnqqD3QNmWc0ZMTaYBNujneUCOokzjtVDEj6ezaEZbPdel9foG2op9HfnDtMqEyHCKhD8E3PqBJ5-mk3HNRZG0fNBt-D-A2dRic6KWrlxTYiE6XAbp1aQ0_G9ii_p9WyQ","p":"zlF2rCHlsv7Ts-EXoLlDn_etK6OOm7bvmUODRMUm06Hg3a9huFfPUdVSRiqHNz3sjeeeACRFXSprzNvbJtLpDdXa_vnrEQojLfQyz4PIbCgojWQTLm4J6F3tdS7s-DpyREDdylIbsm_mZHrlTt07OXjeuz6kutJgowWGen13sQM","q":"0XI2jxrkBdcqtj0K5NWRYjizZzfRw7AHladqKW_QoG-qvdhiCkg040fPKDVFLVV-IKe2QmtpfJCnu1znJbWuA40gITANwyQkE2XR01HsoVvvdHH7JPFEIvGp0ebz20UFZeYiV3Vl8bVTk1J865qwKptQp219nZR5-bifclKJAUM","qi":"O04G3Kr0i0UKWcM5VlpJyzhqbEwenijrDmqGS2d12nYVCOuUvMLwaaKzSNiNimz_ERcLv5k-v4wOtpQNknV2DkC97I-iaPVZxR3msMBtY5eU5qrO_Mz5ZTms-ldPeA9RTpJASQ1x-FSRnDpeYDbVWUJLaewTON6oVMercb_vX2E"}`

	privKey, err := jwk.ParseKey([]byte(privKeySrc))
	if err != nil {
		fmt.Printf("failed to create JWK from raw key: %s\n", err)
		return
	}

	enc.Encode(privKey)

	fmt.Println("JWTにJWS署名を付与")

	// privKeyで渡している値はjwk.Keyですが、rsa.PrivateKeyでも問題ありません。
	signed, err := jwt.Sign(token, jwt.WithKey(jwa.RS256, privKey))
	if err != nil {
		fmt.Printf("failed to sign JWT: %s\n", err)
		return
	}
	fmt.Printf("%s\n", signed)

	fmt.Println("署名を検証した結果")

	// privKeyから公開鍵を作成
	// 実際に署名をする際はこの鍵をJWKとしてインターネットに公開
	// しておけば、誰でも検証が可能となります。
	pubKey, _ := privKey.PublicKey()

	// 検証
	// jws.Parseを使っても検証は可能。jwt.Parseをするとトークンの
	// パースまで行う
	verifiedToken, err := jwt.Parse(signed, jwt.WithKey(jwa.RS256, pubKey))
	if err != nil {
		fmt.Printf("failed to verify JWT: %s\n", err)
		return
	}

	fmt.Println("検証成功")

	fmt.Println("jwt.Parseから得られたJWT")
	enc.Encode(verifiedToken)

	// ここから先の暗号化はコードを実行する度に出力が変わるので、静的な出力結果を
	// 期待するGoのExample形式とは相性が悪いためコメントアウトしてあります。
	//コメントをはずすと正しく動作してもテスト結果が失敗したかのように
	// 見えますのでご注意ください

	/*
		fmt.Println("JWSメッセージをさらに暗号化")
		encrypted, err := jwe.Encrypt(signed, jwe.WithKey(jwa.RSA_OAEP, pubKey))
		if err != nil {
			fmt.Printf("failed to encrypt JWS/JWT: %s\n", err)
			return
		}

		fmt.Printf("%s\n", encrypted)

		decrypted, err := jwe.Decrypt(encrypted, jwe.WithKey(jwa.RSA_OAEP, pubKey))
		if err != nil {
			fmt.Printf("failed to decrypt JWE: %s\n", err)
			return
		}

		// decryptedの値はsignedと同じになるので、jwt.Parse等でパースします。
		_ = decrypted
	*/

	// OUTPUT:
	// JWTの作成
	// {"aud":"wdpress-readers","iat":1679901069,"iss":"gihyo.jp/wd-press/when-ingo","sub":"sample"}
	// JWKの作成
	// {"d":"QecTOvJSEUkdmi-4oolu5UM2jbcsgdZgYou5OarsbubLaQOBOb3u38fr_eWkIiB4x0atVScC6PD2nldQlOcE9mQhhyumoSAYGZ-u4yU7PIrn23hYw7poLXzSbxeXtrRNSE6CMo36IMZFTpTA7Km5ur8wRVLYyjpz1YZG7zZiHlxeCH9xyuvrScCefFwI96fL0MrX56x0-ZMXX65pByZLtAYJmKFuep7u8UlJaR9f0PIlL78JkioveqFPrvzxI2LN3_BGagHDgC75vvC0m9DqfJ3xCoLoasFehPPrfuKk2mZm28Yp4mth7mL7zUgoe0UhHVEtipmIAdqo_7INZOg1TQ","dp":"sKtTWmygW9kYAUUzTV63SKj2sZKZkD9jBjOomV_VMo3AlzQSEp-mmOV5UTojlStCf_Ug9_AXMYMc0kxFagZaqI1CvOnW3z12CGi-hbHMEsmEiCZ0wmJZu4-KwbiQFLM_m7CL-y66Jt7pQ4t1BkYs67bTyzUGLC_vrkkR8Zu0vW8","dq":"hE7aGFysXuQlnynAj_458QpQiovAfxXcNLWlYre32yGvgm4GwOgMQOdD_epo4NJ48rf_csXXxVC8JKfzfObgHm4wNcbdasCR6Y8QlaXTDbu98JlwRU4D9O8kj-yNb56SYmSWnRkUZTRer1oUf-EgGEo1EQ8ho0IAK7htQRLSaXE","e":"AQAB","kty":"RSA","n":"qMyOIeSJRAlQSKZux2HGIzgOfyRPh87MF8Xtn-SG7UC0utgSdjtL2HzmsgG4nxY696kKu9a9Ww4Sn1w53x6LTgpibElqT87Jih4zUHjYXhv1TF7uOMKyq2_i0wcFiSjpH901FmxSY6yy0b9DHkWZaaIAoBzLttXURw6kW71hJZSNeUYP2V1WzYdRATcirLRyXKBVQDYWcgq2cblH9oxT2GnqqD3QNmWc0ZMTaYBNujneUCOokzjtVDEj6ezaEZbPdel9foG2op9HfnDtMqEyHCKhD8E3PqBJ5-mk3HNRZG0fNBt-D-A2dRic6KWrlxTYiE6XAbp1aQ0_G9ii_p9WyQ","p":"zlF2rCHlsv7Ts-EXoLlDn_etK6OOm7bvmUODRMUm06Hg3a9huFfPUdVSRiqHNz3sjeeeACRFXSprzNvbJtLpDdXa_vnrEQojLfQyz4PIbCgojWQTLm4J6F3tdS7s-DpyREDdylIbsm_mZHrlTt07OXjeuz6kutJgowWGen13sQM","q":"0XI2jxrkBdcqtj0K5NWRYjizZzfRw7AHladqKW_QoG-qvdhiCkg040fPKDVFLVV-IKe2QmtpfJCnu1znJbWuA40gITANwyQkE2XR01HsoVvvdHH7JPFEIvGp0ebz20UFZeYiV3Vl8bVTk1J865qwKptQp219nZR5-bifclKJAUM","qi":"O04G3Kr0i0UKWcM5VlpJyzhqbEwenijrDmqGS2d12nYVCOuUvMLwaaKzSNiNimz_ERcLv5k-v4wOtpQNknV2DkC97I-iaPVZxR3msMBtY5eU5qrO_Mz5ZTms-ldPeA9RTpJASQ1x-FSRnDpeYDbVWUJLaewTON6oVMercb_vX2E"}
	// JWTにJWS署名を付与
	// eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJhdWQiOiJ3ZHByZXNzLXJlYWRlcnMiLCJpYXQiOjE2Nzk5MDEwNjksImlzcyI6ImdpaHlvLmpwL3dkLXByZXNzL3doZW4taW5nbyIsInN1YiI6InNhbXBsZSJ9.URYAPuS3tuSzTSitVLdyNUYGpkm6Wm7I_-KIpUxave-P7XKPMjr0Er32qE--f3vdWtltz5_wtN3YxP15Rt8a8KqVje1faRM2esLLPWLJg0lUD1ZaUHTaK-gOHUs0cHGi4vxu8pNNON6uprqdi0P8HX2ztEOCCHLxi6jTIUP4APF-AoIzGBevl0gv5xOxERpMwO7nVrJfhorjxczyU-lMsWrykjNqaEDfHeeqqaK6NgYW9reYyJdhPObKQx_uTjxT3FTUF3QDvoXSbido5DhebSz_g71yCL7qEmwbzOt58orJfxxwyUgiALrG7O3BrlqkPJjFvbW3zhAFKrIitJt3rQ
	// 署名を検証した結果
	// 検証成功
	// jwt.Parseから得られたJWT
	// {"aud":"wdpress-readers","iat":1679901069,"iss":"gihyo.jp/wd-press/when-ingo","sub":"sample"}
}
